#----------------------------------------------------------------
# Copyright (c) 2002 Benjamin Crowell, all rights reserved.
#
# This software is available under version 2 of the GPL license.
# The software is copyrighted, and you must agree to the
# license in order to have permission to copy it. The full
# text of the license is given in the file titled Copying.
#
#----------------------------------------------------------------


use strict;
use Portable;

package Preferences;

sub new {
  my $class = shift;
  my %args = (
                GB=>{},
                @_
                );
  my $self = {};
  bless($self,$class);

  my %dir = (
    'traditional_unix'=>'~',
    'macos_x'=>'~/Library/Preferences',
    'windows'=>'MY_DOCUMENTS_DRIVE:/My Documents'
  );

  my %name = (
    'traditional_unix'=>'.OpenGrade_prefs',
    'macos_x'=>'OpenGrade_prefs',
    'windows'=>'ogr.prf'
  );


  my $os = Portable::os_type();
  if (!exists $dir{$os}) {$os = 'traditional_unix'}

  my $dir = $dir{$os};
  # On a Windows machine, the My Documents folder may be on C, or D, or God knows what. Try to find it:
  if ($os eq 'windows') {
    my $dir_for_my_documents;
    my @drives = ('C','D','E'); # Avoid A and B, because then it makes the floppy drive grind. This should work for must users.
    foreach my $drive(@drives) {
      my $try = $dir;
      $try =~ s/MY_DOCUMENTS_DRIVE/$drive/;
      if (-e $try && -d $try) {$dir_for_my_documents = $try}
    }
    $dir = $dir_for_my_documents;
  }

  my ($err,$file) = Portable::do_glob_one($dir);
  $file = $file."/".$name{$os};

  if (!-e $file) {
    open F,">$file" or return $self;
    close F;
  }

  open F,"<$file" or return $self;
  close F or return $self;
  $self->{FILE} = $file;

  return $self;
}

sub get {
  my $self = shift;
  my $what = shift;
  local $/; # slurp whole file
  open F,"<".$self->{FILE} or return '';
  my $stuff = <F>;
  close F;
  if ($stuff =~ m/$what=\"([^\"]+)\"/) {
    return $1;
  }
  return '';
}

sub set {
  my $self = shift;
  my $what = shift;
  my $value = shift;
  local $/; # slurp whole file
  open F,"<".$self->{FILE} or return '';
  my $stuff = <F>;
  close F;
  $stuff =~ s/$what=\"[^\"]*\"\n?//g;
  $stuff = $stuff . "$what=\"$value\"\n";
  open F,">".$self->{FILE} or return '';
  print F $stuff;
  close F;
  return 1;
}


1;
